/*  smplayer, GUI front-end for mplayer.
    Copyright (C) 2007 Ricardo Villalba <rvm@escomposlinux.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "preferences.h"
#include "constants.h"
#include "global.h"
#include <qsettings.h>
#include <qfileinfo.h>
#include <qregexp.h>

Preferences::Preferences() {
	reset();
	load();
}

Preferences::~Preferences() {
	save();
}

void Preferences::reset() {
#ifdef Q_OS_WIN
	mplayer_bin= "mplayer/mplayer.exe";
#else
	mplayer_bin = "mplayer";
#endif

	QFileInfo fi(mplayer_bin);
	if (fi.exists()) {
		mplayer_bin = fi.absFilePath();
		qDebug("mplayer_bin: '%s'", mplayer_bin.utf8().data());
	}

	use_fontconfig = FALSE;
	use_ass_subtitles = FALSE;
	font_file = "";
	font_name = "";
	subcp = "ISO-8859-1";
	font_autoscale = 1;
	font_textscale = 5;
	autoload_sub = TRUE;
	subfuzziness = 1;
	ass_color = 0xFFFF00;
    ass_border_color = 0x000000;

	osd = OSD_SEEK;

	vo = ""; 
	ao = "";

#ifdef Q_OS_WIN
	dvd_device="E:";
	cdrom_device="E:";
#else
	dvd_device="/dev/dvd";
	cdrom_device="/dev/cdrom";
#endif

	// MPlayer 1.0rc1 require restart, new versions don't
	audio_change_requires_restart = FALSE; // Now we need a svn mplayer
	fast_chapter_change = FALSE;

	cache = 2000;

	use_mplayer_window = FALSE;

	monitor_aspect=""; // Autodetect
	
	latest_dir="";
	last_url="";
	last_dvd_directory="";

	mplayer_verbose="";

	resize_method = RESIZE_ALWAYS;
	disable_screensaver = TRUE;

	screenshot_directory="";
	subtitles_on_screenshots = TRUE;

	use_soft_video_eq = FALSE;
	use_soft_vol = FALSE;

	initial_volume = 40;

	mplayer_additional_options="";
    mplayer_additional_video_filters="";
    mplayer_additional_audio_filters="";

	wheel_seek = 30;

	priority = PRIORITY_ABOVENORMAL; // Option only for windows
	frame_drop = FALSE;
	hard_frame_drop = FALSE;
	autosync = FALSE;
	autosync_factor = 100;

	dont_remember_media_settings = FALSE;
	dont_remember_time_pos = FALSE;

#if STYLE_SWITCHING
	style="";
#endif

	fullscreen = FALSE;
	start_in_fullscreen = FALSE;

	compact_mode = FALSE;

	show_frame_counter = FALSE;

	autoq = 6;

	use_single_instance = FALSE;
	connection_port = 8000;
}

void Preferences::save() {
	qDebug("Preferences::save");

	QSettings * set = settings;

	set->beginGroup( "preferences");

	set->writeEntry("mplayer_bin", mplayer_bin);
	set->writeEntry("use_fontconfig", use_fontconfig);
	set->writeEntry("font_file", font_file);
	set->writeEntry("font_name", font_name);
	set->writeEntry("font_autoscale", font_autoscale);
	set->writeEntry("font_textscale", font_textscale);
	set->writeEntry("subcp", subcp);
	set->writeEntry("use_ass_subtitles", use_ass_subtitles);
	set->writeEntry("autoload_sub", autoload_sub);
	set->writeEntry("subfuzziness", subfuzziness);
	set->writeEntry("ass_color", (int) ass_color);
	set->writeEntry("ass_border_color", (int) ass_border_color);

	set->writeEntry("osd", osd);
	set->writeEntry("vo", vo);
	set->writeEntry("ao", ao);

	set->writeEntry("audio_change_requires_restart", audio_change_requires_restart);
	set->writeEntry("fast_chapter_change", fast_chapter_change);

	set->writeEntry("dvd_device", dvd_device);
	set->writeEntry("cdrom_device", cdrom_device);

	set->writeEntry("cache", cache);
	set->writeEntry("use_mplayer_window", use_mplayer_window);

	set->writeEntry("monitor_aspect", monitor_aspect);

	set->writeEntry("latest_dir", latest_dir);
	set->writeEntry("last_url", last_url);
	set->writeEntry("last_dvd_directory", last_dvd_directory);

	set->writeEntry("mplayer_verbose", mplayer_verbose);
	set->writeEntry("resize_method", resize_method);
	set->writeEntry("disable_screensaver", disable_screensaver);

	set->writeEntry("screenshot_directory", screenshot_directory);
	set->writeEntry("subtitles_on_screenshots", subtitles_on_screenshots);

	set->writeEntry("use_soft_video_eq", use_soft_video_eq);
	set->writeEntry("use_soft_vol", use_soft_vol);

	set->writeEntry("initial_volume", initial_volume);

	set->writeEntry("mplayer_additional_options", mplayer_additional_options);
	set->writeEntry("mplayer_additional_video_filters", mplayer_additional_video_filters);
	set->writeEntry("mplayer_additional_audio_filters", mplayer_additional_audio_filters);

	set->writeEntry("wheel_seek", wheel_seek);

	set->writeEntry("priority", priority);
	set->writeEntry("frame_drop", frame_drop);
	set->writeEntry("hard_frame_drop", hard_frame_drop);
	set->writeEntry("autosync", autosync);
	set->writeEntry("autosync_factor", autosync_factor);

	set->writeEntry("dont_remember_media_settings", dont_remember_media_settings);
	set->writeEntry("dont_remember_time_pos", dont_remember_time_pos);

#if STYLE_SWITCHING
	set->writeEntry("style", style);
#endif

	set->writeEntry("fullscreen", fullscreen);
	set->writeEntry("start_in_fullscreen", start_in_fullscreen);

	set->writeEntry("compact_mode", compact_mode);

	set->writeEntry("show_frame_counter", show_frame_counter);

	set->writeEntry("autoq", autoq);

	set->writeEntry("use_single_instance", use_single_instance);
	set->writeEntry("connection_port", connection_port);

	set->endGroup();
}

void Preferences::load() {
	qDebug("Preferences::load");

	QSettings * set = settings;

	set->beginGroup( "preferences");

	mplayer_bin = set->readEntry("mplayer_bin", mplayer_bin);

	use_fontconfig = set->readBoolEntry("use_fontconfig", use_fontconfig);
	font_file = set->readEntry("font_file", font_file);
	font_name = set->readEntry("font_name", font_name);
	font_autoscale = set->readNumEntry("font_autoscale", font_autoscale);
	font_textscale = set->readNumEntry("font_textscale", font_textscale);
	subcp = set->readEntry("subcp", subcp);
	subfuzziness = set->readNumEntry("subfuzziness", subfuzziness);
	use_ass_subtitles = set->readBoolEntry("use_ass_subtitles", use_ass_subtitles);
	autoload_sub = set->readBoolEntry("autoload_sub", autoload_sub);
	ass_color = set->readNumEntry("ass_color", ass_color);
	ass_border_color = set->readNumEntry("ass_border_color", ass_border_color);

	osd = set->readNumEntry("osd", osd);
	vo = set->readEntry("vo", vo);
	ao = set->readEntry("ao", ao);

	audio_change_requires_restart = set->readBoolEntry("audio_change_requires_restart", audio_change_requires_restart);
	fast_chapter_change = set->readBoolEntry("fast_chapter_change", fast_chapter_change);

	dvd_device = set->readEntry("dvd_device", dvd_device);
	cdrom_device = set->readEntry("cdrom_device", cdrom_device);

	cache = set->readNumEntry("cache", cache);
	use_mplayer_window = set->readBoolEntry("use_mplayer_window", use_mplayer_window);

	monitor_aspect = set->readEntry("monitor_aspect", monitor_aspect);
		
	latest_dir = set->readEntry("latest_dir", latest_dir);
	last_url = set->readEntry("last_url", last_url);
	last_dvd_directory = set->readEntry("last_dvd_directory", last_dvd_directory);

	mplayer_verbose = set->readEntry("mplayer_verbose", mplayer_verbose);
	resize_method = set->readNumEntry("resize_method", resize_method);
	disable_screensaver = set->readBoolEntry("disable_screensaver", disable_screensaver);

	screenshot_directory = set->readEntry("screenshot_directory", screenshot_directory);
	subtitles_on_screenshots = set->readBoolEntry("subtitles_on_screenshots", subtitles_on_screenshots);

	use_soft_video_eq = set->readBoolEntry("use_soft_video_eq", use_soft_video_eq);
	use_soft_vol = set->readBoolEntry("use_soft_vol", use_soft_vol);

	initial_volume = set->readNumEntry("initial_volume", initial_volume);

	mplayer_additional_options = set->readEntry("mplayer_additional_options", mplayer_additional_options);
	mplayer_additional_video_filters = set->readEntry("mplayer_additional_video_filters", mplayer_additional_video_filters);
	mplayer_additional_audio_filters = set->readEntry("mplayer_additional_audio_filters", mplayer_additional_audio_filters);

	wheel_seek = set->readNumEntry("wheel_seek", wheel_seek);

	priority = set->readNumEntry("priority", priority);
	frame_drop = set->readBoolEntry("frame_drop", frame_drop);
	hard_frame_drop = set->readBoolEntry("hard_frame_drop", hard_frame_drop);
	autosync = set->readBoolEntry("autosync", autosync);
	autosync_factor = set->readNumEntry("autosync_factor", autosync_factor);

	dont_remember_media_settings = set->readBoolEntry("dont_remember_media_settings", dont_remember_media_settings);
	dont_remember_time_pos = set->readBoolEntry("dont_remember_time_pos", dont_remember_time_pos);

#if STYLE_SWITCHING
	style = set->readEntry("style", style);
#endif

	fullscreen = set->readBoolEntry("fullscreen", fullscreen);
	start_in_fullscreen = set->readBoolEntry("start_in_fullscreen", start_in_fullscreen);

	compact_mode = set->readBoolEntry("compact_mode", compact_mode);

	show_frame_counter = set->readBoolEntry("show_frame_counter", show_frame_counter);

	autoq = set->readNumEntry("autoq", autoq);

	use_single_instance = set->readBoolEntry("use_single_instance", use_single_instance);
	connection_port = set->readNumEntry("connection_port", connection_port);

	set->endGroup();

	QFileInfo fi(mplayer_bin);
	if (fi.exists()) {
		mplayer_bin = fi.absFilePath();
		qDebug("mplayer_bin: '%s'", mplayer_bin.utf8().data());
	}
}

double Preferences::monitor_aspect_double() {
	qDebug("Preferences::monitor_aspect_double");

	QRegExp exp("(\\d+)[:/](\\d+)");
	if (exp.search( monitor_aspect ) != -1) {
		int w = exp.cap(1).toInt();
		int h = exp.cap(2).toInt();
		qDebug(" monitor_aspect parsed successfully: %d:%d", w, h);
		return (double) w/h;
	}

	bool ok;
	double res = monitor_aspect.toDouble(&ok);
	if (ok) {
		qDebug(" monitor_aspect parsed successfully: %f", res);
		return res;
	} else {
		qDebug(" warning: monitor_aspect couldn't be parsed!");
        qDebug(" monitor_aspect set to 0");
		return 0;
	}
}
